library IEEE;
use IEEE.std_logic_1164.all;
entity RESET_GEN is
   port(CLK            : in std_logic;
        RESN_GEN       : out std_logic);
end RESET_GEN;

architecture RTL of RESET_GEN is
   signal RESN_GEN_0 : std_logic;
   signal CNT : integer range 0 to 3;
begin

   process(CLK)
   begin
      if CLK'event and CLK = '0' then
         if CNT = 3 then
            RESN_GEN_0 <= '1';
         else
            RESN_GEN_0 <= '0';
         end if;
      end if;
   end process;

   process(CLK)
   begin
      if CLK'event and CLK = '1' then
         if CNT < 3 then
            CNT <= CNT + 1;
         end if;
      end if;
   end process;

   RESN_GEN <= RESN_GEN_0;

end RTL;

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;

entity fifo_8_16 is
   port (CLK, RESN:       IN  std_logic;
         read_en:         IN  std_logic;
         write_en:        IN  std_logic;
         data_in:         IN  std_logic_vector(15 downto 0);
         fifo_resetn:     IN  std_logic;
         data_out:        OUT std_logic_vector(15 downto 0);
         fifo_full:       OUT std_logic;
         fifo_empty:      OUT std_logic);
END fifo_8_16;

architecture rtl of fifo_8_16 is

   type FIFO_MEM is array (7 downto 0) of std_logic_vector(15 downto 0);
   signal FIFO : FIFO_MEM;
   signal FIFO_CNT : integer range 0 to 8;
   signal fifo_empty_int : std_logic;
   signal fifo_full_int : std_logic;
begin

   process(CLK, RESN)
   begin
      if RESN = '0' then
         FIFO_CNT <= 0;
      elsif CLK'event and CLK = '1' then
         if write_en = '1' and fifo_full_int = '0' then
            FIFO_CNT <= FIFO_CNT + 1;
         elsif read_en = '1' and fifo_empty_int = '0' then
            FIFO_CNT <= FIFO_CNT - 1;
         end if;
      end if;
   end process;
   
   fifo_empty_int <= '1' when FIFO_CNT = 0 else '0';
   fifo_empty     <= fifo_empty_int;
   fifo_full_int  <= '1' when FIFO_CNT = 8 else '0';
   fifo_full      <= fifo_full_int;

   process(CLK, RESN)
   begin
      if RESN = '0' then
         FIFO <= (others => (others => '0'));
      elsif CLK'event and CLK = '1' then
         if fifo_resetn = '0' then
            FIFO <= (others => (others => '0'));
         elsif write_en = '1' and fifo_empty_int = '1' then 
            FIFO(0) <= data_in;
         elsif write_en = '1' and fifo_full_int = '0' then
           case FIFO_CNT is
              when 0 => FIFO(0) <= data_in;
              when 1 => FIFO(1) <= data_in;
              when 2 => FIFO(2) <= data_in;
              when 3 => FIFO(3) <= data_in;
              when 4 => FIFO(4) <= data_in;
              when 5 => FIFO(5) <= data_in;
              when 6 => FIFO(6) <= data_in;
              when 7 => FIFO(7) <= data_in;
              when others => FIFO(7) <= data_in;
            end case;
         end if;
      end if;
   end process;

   process(read_en, fifo_empty_int, FIFO, FIFO_CNT)
   begin
      if read_en = '1' and fifo_empty_int = '0' then
         case FIFO_CNT is
           when 8 => data_out <= FIFO(0);
           when 7 => data_out <= FIFO(1);
           when 6 => data_out <= FIFO(2);
           when 5 => data_out <= FIFO(3);
           when 4 => data_out <= FIFO(4);
           when 3 => data_out <= FIFO(5);
           when 2 => data_out <= FIFO(6);
           when 1 => data_out <= FIFO(7);
           when others => data_out <= FIFO(7);
         end case;
      else
         data_out <= (others => '0');
      end if;
   end process;


END rtl;

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;

entity FIFO_CTRL is
   port (CLK, RESN       : in std_logic;
         SINC3_DATA_OUT  : in std_logic_vector(15 downto 0);
         SINC3_DONE      : in std_logic;
         READ_EN_0       : out std_logic;
         READ_EN_1       : out std_logic;
         WRITE_EN_0      : out std_logic;
         WRITE_EN_1      : out std_logic;
         FIFO_DATA_IN_0  : out std_logic_vector(15 downto 0);
         FIFO_DATA_IN_1  : out std_logic_vector(15 downto 0);
         FIFO_DATA_OUT_0 : in std_logic_vector(15 downto 0);
         FIFO_DATA_OUT_1 : in std_logic_vector(15 downto 0);
         FIFO_RESETN_0   : out std_logic;
         FIFO_RESETN_1   : out std_logic;
         FIFO_FULL_0     : in std_logic;
         FIFO_EMPTY_0    : in std_logic;
         FIFO_FULL_1     : in std_logic;
         FIFO_EMPTY_1    : in std_logic;
         SPI_DATA_OUT    : out std_logic_vector(15 downto 0);
         SPI_EN          : out std_logic;
         SPI_EN_DELAY    : out std_logic;   -- Added for SPI interface (Badde)
         SPI_BUSY        : in std_logic;
         DRDY            : out std_logic;
         FS              : out std_logic);
END FIFO_CTRL;


architecture RTL of FIFO_CTRL is 
   type FIFO_FSM is (WAIT_FIFO_0_FULL, START_FIFO_0_SPI, WAIT_FIFO_1_FULL, 
                     START_FIFO_1_SPI, WAIT_FOR_0_SPI_FINISH, WAIT_FOR_1_SPI_FINISH, 
                     WAIT_0_CYCLE, WAIT_1_CYCLE);
   signal CURRENT_STATE, NEXT_STATE : FIFO_FSM;
   signal READ_EN_0_INT, READ_EN_1_INT : std_logic;
   signal SPI_EN_PULSE, SPI_EN_PULSE_SYNC, SPI_EN_INT, SPI_EN_DELAY_INT : std_logic;

begin
   process(CLK, RESN)
   begin
      if RESN = '0' then
         CURRENT_STATE <= WAIT_FIFO_0_FULL;
      elsif CLK'event and CLK = '1' then
         CURRENT_STATE <= NEXT_STATE;
      end if;
   end process;

   process(CURRENT_STATE, FIFO_FULL_0, FIFO_FULL_1, FIFO_EMPTY_0, FIFO_EMPTY_1,
          SINC3_DONE, SPI_BUSY, SINC3_DATA_OUT)
   begin
      FIFO_RESETN_0 <= '1';
      FIFO_RESETN_1 <= '1';
      FIFO_DATA_IN_0 <= (others => '0');
      FIFO_DATA_IN_1 <= (others => '0');
      WRITE_EN_0     <= '0';
      WRITE_EN_1     <= '0';
      READ_EN_0_INT  <= '0';
      READ_EN_1_INT  <= '0';
      NEXT_STATE     <= CURRENT_STATE;
      SPI_EN_INT     <= '0';
      DRDY           <= '0';
      case CURRENT_STATE is
         when WAIT_FIFO_0_FULL =>
            if FIFO_FULL_0 = '1' and FIFO_EMPTY_1 = '1' and SPI_BUSY = '0' then
               NEXT_STATE <= START_FIFO_0_SPI;
               SPI_EN_INT <= '1';
               DRDY   <= '1';
            end if;
            if SINC3_DONE = '1' then
               WRITE_EN_0     <= '1';
               FIFO_DATA_IN_0 <= SINC3_DATA_OUT;
            end if;

         when START_FIFO_0_SPI => 
            SPI_EN_INT        <= '1';
            READ_EN_0_INT <= '1';
            NEXT_STATE    <= WAIT_FOR_0_SPI_FINISH;

         when WAIT_FOR_0_SPI_FINISH => 
            SPI_EN_INT         <= '1';
            if SPI_BUSY = '0' then
               SPI_EN_INT       <= '0';
               NEXT_STATE <= WAIT_0_CYCLE;
               if FIFO_EMPTY_0 = '1' then
                  FIFO_RESETN_0 <= '0';
                  NEXT_STATE    <= WAIT_FIFO_1_FULL;
               end if;
            end if;

         when WAIT_0_CYCLE =>
            SPI_EN_INT       <= '0';
            NEXT_STATE <= START_FIFO_0_SPI;


         when WAIT_FIFO_1_FULL =>
            if FIFO_FULL_1 = '1' and FIFO_EMPTY_0  = '1' and SPI_BUSY = '0' then
               NEXT_STATE <= START_FIFO_1_SPI;
               SPI_EN_INT <= '1';
               DRDY   <= '1';
            end if;
            if SINC3_DONE = '1' then
               WRITE_EN_1     <= '1';
               FIFO_DATA_IN_1 <= SINC3_DATA_OUT;
            end if;

         when START_FIFO_1_SPI => 
            SPI_EN_INT        <= '1';
            READ_EN_1_INT <= '1';
            NEXT_STATE <= WAIT_FOR_1_SPI_FINISH;

         when WAIT_FOR_1_SPI_FINISH => 
            SPI_EN_INT       <= '1';
            if SPI_BUSY = '0' then
               SPI_EN_INT       <= '0';
               NEXT_STATE <= WAIT_1_CYCLE;
               if FIFO_EMPTY_1 = '1' then
                  FIFO_RESETN_1 <= '0';
                  NEXT_STATE    <= WAIT_FIFO_0_FULL;
               end if;
            end if;

         when WAIT_1_CYCLE =>
            SPI_EN_INT       <= '0';
            NEXT_STATE <= START_FIFO_1_SPI;

         when others => NEXT_STATE <= WAIT_FIFO_0_FULL;
      end case;
   end process;

   process(CLK, RESN)
   begin
      if RESN = '0' then
         SPI_DATA_OUT <= (others => '0');
      elsif CLK'event and CLK = '1' then
         if READ_EN_0_INT = '1' then
            SPI_DATA_OUT <= FIFO_DATA_OUT_0;
         elsif READ_EN_1_INT = '1' then
            SPI_DATA_OUT <= FIFO_DATA_OUT_1;
         end if;
      end if;
   end process;

   READ_EN_0 <= READ_EN_0_INT;
   READ_EN_1 <= READ_EN_1_INT;
   
   SPI_EN_PULSE <= (SPI_EN_DELAY_INT XOR SPI_EN_INT) AND SPI_EN_INT;
   SPI_EN       <= SPI_EN_INT;
   SPI_EN_DELAY <= SPI_EN_DELAY_INT;     -- Route this signal to SPI (Badde)
   
   process(CLK, RESN)
   begin
      if RESN = '0' then
         SPI_EN_PULSE_SYNC <= '0';
         SPI_EN_DELAY_INT <= '0';
      elsif CLK'event and CLK = '1' then
         SPI_EN_DELAY_INT <= SPI_EN_INT;
         SPI_EN_PULSE_SYNC <= SPI_EN_PULSE;
      end if;
   end process;

   FS <= SPI_EN_PULSE_SYNC;
   
   


end RTL;

library IEEE;
use IEEE.std_logic_1164.all;

entity TOP_FIFO is
   port(RESN, CLK      : in std_logic;
        SINC3_OUT      : in std_logic_vector(15 downto 0);
        SINC3_DONE     : in std_logic;
        SPI_DATA_OUT   : out std_logic_vector(15 downto 0);
        SPI_EN         : out std_logic;
        SPI_EN_DELAY   : out std_logic;
        SPI_BUSY       : in std_logic;
        DRDY           : out std_logic;
        FS             : out std_logic;
        READ_EN        : out std_logic;
        WRITE_EN       : out std_logic);
end TOP_FIFO;

architecture RTL of TOP_FIFO is

   signal FIFO_DATA_IN_0, FIFO_DATA_OUT_0 : std_logic_vector(15 downto 0); 
   signal READ_EN_0, WRITE_EN_0, FIFO_RESETN_0, FIFO_FULL_0, FIFO_EMPTY_0 : std_logic;
   signal READ_EN_1, WRITE_EN_1, FIFO_RESETN_1, FIFO_FULL_1, FIFO_EMPTY_1 : std_logic;
   signal FIFO_DATA_IN_1, FIFO_DATA_OUT_1 : std_logic_vector(15 downto 0);



   component FIFO_8_16 
      port (CLK, RESN:       IN  std_logic;
            read_en:         IN  std_logic;
            write_en:        IN  std_logic;
            data_in:         IN  std_logic_vector(15 downto 0);
            fifo_resetn:     IN  std_logic;
            data_out:        OUT std_logic_vector(15 downto 0);
            fifo_full:       OUT std_logic;
            fifo_empty:      OUT std_logic);
   end component;

   component FIFO_CTRL
      port (CLK, RESN       : in std_logic;
            SINC3_DATA_OUT  : in std_logic_vector(15 downto 0);
            SINC3_DONE      : in std_logic;
            READ_EN_0       : out std_logic;
            READ_EN_1       : out std_logic;
            WRITE_EN_0      : out std_logic;
            WRITE_EN_1      : out std_logic;
            FIFO_DATA_IN_0  : out std_logic_vector(15 downto 0);
            FIFO_DATA_IN_1  : out std_logic_vector(15 downto 0);
            FIFO_DATA_OUT_0 : in std_logic_vector(15 downto 0);
            FIFO_DATA_OUT_1 : in std_logic_vector(15 downto 0);
            FIFO_RESETN_0   : out std_logic;
            FIFO_RESETN_1   : out std_logic;
            FIFO_FULL_0     : in std_logic;
            FIFO_EMPTY_0    : in std_logic;
            FIFO_FULL_1     : in std_logic;
            FIFO_EMPTY_1    : in std_logic;
            SPI_DATA_OUT    : out std_logic_vector(15 downto 0);
            SPI_EN          : out std_logic;
            SPI_EN_DELAY    : out std_logic;
            SPI_BUSY        : in std_logic;
            DRDY            : out std_logic;
            FS              : out std_logic);
   end component;

begin

  FIFO_LOW_UNIT : FIFO_8_16 
     port map(CLK         => CLK,
              RESN        => RESN,
              READ_EN     => READ_EN_0,
              WRITE_EN    => WRITE_EN_0,
              DATA_IN     => FIFO_DATA_IN_0,
              FIFO_RESETN => FIFO_RESETN_0,
              DATA_OUT    => FIFO_DATA_OUT_0,
              FIFO_FULL   => FIFO_FULL_0,
              FIFO_EMPTY  => FIFO_EMPTY_0);

  FIFO_HIGH_UNIT : FIFO_8_16 
     port map(CLK         => CLK,
              RESN        => RESN,
              READ_EN     => READ_EN_1,
              WRITE_EN    => WRITE_EN_1,
              DATA_IN     => FIFO_DATA_IN_1,
              FIFO_RESETN => FIFO_RESETN_1,
              DATA_OUT    => FIFO_DATA_OUT_1,
              FIFO_FULL   => FIFO_FULL_1,
              FIFO_EMPTY  => FIFO_EMPTY_1);

  FIFO_CRTL_UNIT : FIFO_CTRL
     port map(CLK              => CLK,
              RESN             => RESN,
              SINC3_DATA_OUT   => SINC3_OUT,
              SINC3_DONE       => SINC3_DONE,
              READ_EN_0        => READ_EN_0,
              READ_EN_1        => READ_EN_1,
              WRITE_EN_0       => WRITE_EN_0,
              WRITE_EN_1       => WRITE_EN_1,
              FIFO_DATA_IN_0   => FIFO_DATA_IN_0,
              FIFO_DATA_IN_1   => FIFO_DATA_IN_1,
              FIFO_DATA_OUT_0  => FIFO_DATA_OUT_0,
              FIFO_DATA_OUT_1  => FIFO_DATA_OUT_1,
              FIFO_RESETN_0    => FIFO_RESETN_0,
              FIFO_RESETN_1    => FIFO_RESETN_1,
              FIFO_FULL_0      => FIFO_FULL_0,
              FIFO_EMPTY_0     => FIFO_EMPTY_0,
              FIFO_FULL_1      => FIFO_FULL_1,
              FIFO_EMPTY_1     => FIFO_EMPTY_1,
              SPI_DATA_OUT     => SPI_DATA_OUT,
              SPI_EN           => SPI_EN,
              SPI_EN_DELAY     => SPI_EN_DELAY,
              SPI_BUSY         => SPI_BUSY,
              DRDY             => DRDY,
              FS               => FS);                       

  READ_EN  <= READ_EN_0;
  WRITE_EN <= WRITE_EN_0;
  
end RTL;

library ieee;
use ieee.std_logic_1164.all;

entity SPI_MASTER is
   port (CLK, RESN       : in std_logic;
         SPI_DATA_OUT    : in std_logic_vector(15 downto 0);
         SPI_EN          : in std_logic;
         SPI_EN_DELAY    : in std_logic;       -- From FIFO (Badde)
         SPI_BUSY        : out std_logic;
         NOT_SPI_BUSY    : out std_logic;
         SCK             : out std_logic;
         MOSI            : out std_logic;
         LDAC            : out std_logic);
END SPI_MASTER;

architecture RTL of SPI_MASTER is            -- Added a delayed counter (Badde)
   signal COUNTER, COUNTER_DELAY : integer range 0 to 15;
   signal SPI_BUSY_INT , LDAC_int : std_logic;   
   signal CLKn, CLK_GATING, DATA_GATING : std_logic; -- Also data is gated (Badde)
begin

   CLKn <= not CLK;

   process(CLK, RESN)
   begin
      if RESN = '0' then
         SPI_BUSY_INT <= '0';
         COUNTER      <= 15;
         COUNTER_DELAY <= 15;
         DATA_GATING <= '0';
         LDAC_int <= '1';
      elsif CLK'event and CLK = '1' then
         LDAC_int <= not SPI_EN_DELAY;
         COUNTER_DELAY <= COUNTER;
         DATA_GATING <= CLK_GATING;
         if SPI_EN = '1' then 
            SPI_BUSY_INT <= '1';
         end if;
         if COUNTER = 0 then
            COUNTER <= 15;
            SPI_BUSY_INT <= '0';
         elsif SPI_BUSY_INT = '1' then 
            COUNTER <= COUNTER - 1;
         end if;
      end if;
   end process;

   process(CLKn, RESN)        -- Changed the clock gating (Badde)
   begin
      if RESN = '0' then
         CLK_GATING <= '0';
      elsif CLKn'event and CLKn = '1' then
         CLK_GATING <= SPI_BUSY_INT;
      end if;
   end process;

   SPI_BUSY <= SPI_BUSY_INT;
   NOT_SPI_BUSY <= not SPI_EN_DELAY;    -- This is the new SPI FS (Badde) 
   LDAC <= LDAC_int and (not SPI_EN_DELAY);
   SCK <= CLK and CLK_GATING;
   MOSI <= SPI_DATA_OUT(COUNTER_DELAY) when DATA_GATING = '1' else '0'; -- (Badde)
end RTL;

library ieee;
use ieee.std_logic_1164.all;

entity McBSP is
   port (CLK, RESN       : in std_logic;
         SPI_DATA_OUT    : in std_logic_vector(15 downto 0);
         SPI_EN          : in std_logic;
         FS              : out std_logic;     -- Removed SPI_BUSY (Badde)
         SCLK            : out std_logic;
         SDO             : out std_logic;
         LDAC            : out std_logic);
END McBSP;

architecture RTL of McBSP is
   signal counter : integer range 0 to 15;
   signal SPI_BUSY_INT : std_logic;    -- Removed CLK_GATING not used (Badde)
   signal LDAC_int     : std_logic;
begin

   process(CLK, RESN)
   begin
      if RESN = '0' then
         SPI_BUSY_INT <= '0';
         COUNTER      <= 15;
         LDAC_int     <= '1';
      elsif CLK'event and CLK = '1' then
         LDAC_int     <= not SPI_BUSY_INT;
         if SPI_EN = '1' then 
            SPI_BUSY_INT <= '1';
         end if;
         if COUNTER = 0 then
            COUNTER      <= 15;
            SPI_BUSY_INT <= '0';
         elsif SPI_BUSY_INT = '1' then 
            COUNTER <= COUNTER - 1;
         end if;
      end if;
   end process;
                                        -- Removed CLK_GATING process (Badde)
   FS <= not SPI_BUSY_INT;
   LDAC <= not SPI_BUSY_INT and LDAC_int;
   SCLK <= CLK;
   SDO <= SPI_DATA_OUT(COUNTER) when SPI_EN <= '1' else '0';
end RTL;

library ieee;
use ieee.std_logic_1164.all;
entity SINC3_SYNC is
   port (CLK, RESN       : in std_logic;
         SINC3_DATA_OUT  : in std_logic_vector(15 downto 0);
         SINC3_DONE      : in std_logic;
         SINC3_DATA_SYNC : out std_logic_vector(15 downto 0);
         DONE_SYNC       : out std_logic);
end SINC3_SYNC;

architecture RTL of SINC3_SYNC is
   signal DONE_SYNC1, DONE_SYNC2 : std_logic;
begin

   process(CLK, RESN)
   begin
      if RESN = '0' then
         DONE_SYNC1 <= '0';
         DONE_SYNC2 <= '0';
      elsif CLK'event and CLK = '1' then
         DONE_SYNC1 <= SINC3_DONE;
         DONE_SYNC2 <= DONE_SYNC1;
      end if;
   end process;
   
   SINC3_DATA_SYNC <= SINC3_DATA_OUT when DONE_SYNC2 = '1' else "0000000000000000";

   DONE_SYNC <= ((DONE_SYNC2 XOR DONE_SYNC1) AND DONE_SYNC2);
end RTL;

library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_unsigned.all;

entity FLT is
   port(RESN, MOUT, MCLK, CNR : in std_logic;            -- Only 24 bits (Badde)
        CN5                   : out std_logic_vector(23 downto 0));
end FLT;

architecture RTL of FLT is                -- Changed structure of filter (Badde)
  signal Z1     : std_logic_vector(23 downto 0);  
  signal Z2     : std_logic_vector(23 downto 0);
  signal Z3     : std_logic_vector(23 downto 0);
  signal Z4     : std_logic_vector(23 downto 0);
  signal Z5     : std_logic_vector(23 downto 0);
  signal Z6     : std_logic_vector(23 downto 0);
  signal Z7     : std_logic_vector(23 downto 0);
begin

  process(MCLK, RESN)
    begin
      if RESN = '0' then
        Z1 <= (others => '0');
        Z2 <= (others => '0');
        Z3 <= (others => '0');
      elsif MCLK'event and MCLK = '1' then
        Z1 <= Z1 + MOUT;
        Z2 <= Z2 + Z1;
        Z3 <= Z3 + Z2;
      end if;  
    end process;
    
  process(CNR, RESN)
    begin
      if RESN = '0' then
        Z4 <= (others => '0');
        Z5 <= (others => '0');
        Z6 <= (others => '0');
        Z7 <= (others => '0');
      elsif CNR'event and CNR = '1' then
        Z4 <= Z3;  
        Z5 <= Z3 - Z4;  
        Z6 <= Z3 - Z4 - Z5;  
        Z7 <= Z3 - Z4 - Z5 - Z6;  
      end if;  
    end process;
    
  CN5   <= Z7;
    
end RTL;


library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_unsigned.all;

entity TOP_FILTER is
   port(RESN, MCLK  : in std_logic;
        MDAT        : in std_logic;
        M0, M1, M2  : in std_logic;
        SINC3_OUT   : out std_logic_vector(15 downto 0);
        SINC3_DONE  : out std_logic;
        CNR_TEST    : out std_logic);
end TOP_FILTER;

architecture RTL of TOP_FILTER is 

   component FLT
      port(RESN, MOUT, MCLK, CNR : in std_logic;
           CN5                   : out std_logic_vector(23 downto 0));
   end component;

   signal DECIMATION : std_logic_vector(2 downto 0);
   signal LOAD_CNT, COUNTER : std_logic_vector(8 downto 0);
   signal DRDY_INT_DELAY, DRDY_INT : std_logic;
   signal MCLKn : std_logic;
   signal SINC3_OUT_INT : std_logic_vector(23 downto 0);
   signal CNT_EXP : std_logic;
   signal CNR : std_logic;
begin

   MCLKn <= not MCLK;

   -- calculate decimation ratio
   
   DECIMATION <= M2 & M1 & M0;
   
   process(DECIMATION, SINC3_OUT_INT)
   begin
      case DECIMATION is
         when "000" => LOAD_CNT  <= "000000100"; -- 4
                       SINC3_OUT <= SINC3_OUT_INT(15 downto 0);
         when "001" => LOAD_CNT  <= "000001000"; -- 8
                       SINC3_OUT <= SINC3_OUT_INT(15 downto 0);
         when "010" => LOAD_CNT  <= "000010000"; -- 16
                       SINC3_OUT <= SINC3_OUT_INT(15 downto 0);
         when "011" => LOAD_CNT  <= "000100000"; -- 32
                       SINC3_OUT <= SINC3_OUT_INT(15 downto 0);
         when "100" => LOAD_CNT  <= "001000000"; -- 64
                       SINC3_OUT <= SINC3_OUT_INT(17 downto 2);
         when "101" => LOAD_CNT  <= "010000000"; -- 128
                       SINC3_OUT <= SINC3_OUT_INT(20 downto 5);
         when others => LOAD_CNT <= "100000000"; -- 256
                       SINC3_OUT <= SINC3_OUT_INT(23 downto 8);
      end case;
   end process;
   
   CNR_CNT : process(MCLK, RESN)
   begin
      if RESN = '0' then
         COUNTER  <= "000000100";  -- decimation ratio 4
      elsif MCLK'event and MCLK = '1' then
         if COUNTER = "000000000" then
            COUNTER <= LOAD_CNT - 1;
         else
            COUNTER <= COUNTER - 1;
         end if;
      end if;
   end process CNR_CNT;

   CNT_EXP <= '1' when COUNTER = "0000000000000" else '0';

   CNR_GEN : process(MCLKn, RESN)
   begin
      if RESN = '0' then
         CNR <= '0';
      elsif MCLKn'event and MCLKn='1' then
         CNR <= not CNT_EXP;
      end if;
   end process CNR_GEN;

   DRDY_GEN : process(CNR, RESN)
   begin
      if RESN = '0' then
         DRDY_INT <= '0';
      elsif CNR'event and CNR = '1' then
         DRDY_INT <= not DRDY_INT;
      end if;
   end process DRDY_GEN;

   -- DRDY signal changes polarity whenever data is ready
   -- take this information to generate set_irq signal
   -- use FCLKn to sync DRDY_INT
   
   SET_IRQ_GEN : process(RESN, MCLK)
   begin
      if RESN = '0' then
         DRDY_INT_DELAY <= '0';
         SINC3_DONE     <= '0';
      elsif MCLK'event and MCLK = '1' then
         DRDY_INT_DELAY <= DRDY_INT;
         SINC3_DONE     <= DRDY_INT xor DRDY_INT_DELAY;
      end if;
   end process SET_IRQ_GEN;

   FILT_UNIT : FLT port map(RESN => RESN, 
                            MOUT => MDAT, 
                            MCLK => MCLK, 
                            CNR  => CNR, 
                            CN5  => SINC3_OUT_INT);


   CNR_TEST <= CNR;


end RTL;



library IEEE;
use IEEE.std_logic_1164.all;

entity TOP is
   port(RESN, CLK  : in std_logic;
        M0, M1, M2 : in std_logic;
        MODE_SEL   : in std_logic;
        MDAT       : in std_logic;
        MCLK       : in std_logic;
        SCK, SSn   : out std_logic;      -- spi mode signals
        SCLK, FS   : out std_logic;  -- dsp mode signals 
        DRDY, SDO  : out std_logic;
        LDAC       : out std_logic;
        T0, T1, T2, T3, T4, T5, T6, T7 : out std_logic;
        T1_0, T1_1, T1_2, T1_3 : out std_logic);
end TOP;

architecture RTL of TOP is 


   component SINC3_SYNC
      port (CLK, RESN       : in std_logic;
            SINC3_DATA_OUT  : in std_logic_vector(15 downto 0);
            SINC3_DONE      : in std_logic;
            SINC3_DATA_SYNC : out std_logic_vector(15 downto 0);
            DONE_SYNC       : out std_logic);
   end component;


   component TOP_FILTER
      port(RESN, MCLK  : in std_logic;
           MDAT        : in std_logic;
           M0, M1, M2  : in std_logic;
           SINC3_OUT   : out std_logic_vector(15 downto 0);
           SINC3_DONE  : out std_logic;
           CNR_TEST    : out std_logic);
   end component;

   component TOP_FIFO
      port(RESN, CLK      : in std_logic;
           SINC3_OUT      : in std_logic_vector(15 downto 0);
           SINC3_DONE     : in std_logic;
           SPI_DATA_OUT   : out std_logic_vector(15 downto 0);
           SPI_EN         : out std_logic;
           SPI_EN_DELAY   : out std_logic;    -- Added (Badde)
           SPI_BUSY       : in std_logic;
           DRDY           : out std_logic;
           FS             : out std_logic;
           READ_EN        : out std_logic;
           WRITE_EN       : out std_logic);
   end component;

   component SPI_MASTER
      port (CLK, RESN       : in std_logic;
            SPI_DATA_OUT    : in std_logic_vector(15 downto 0);
            SPI_EN          : in std_logic;
            SPI_EN_DELAY    : in std_logic;     -- Added (Badde)
            SPI_BUSY        : out std_logic;
            NOT_SPI_BUSY    : out std_logic;
            SCK             : out std_logic;
            MOSI            : out std_logic;
            LDAC            : out std_logic);
   end component;

      component McBSP
      port (CLK, RESN       : in std_logic;
            SPI_DATA_OUT    : in std_logic_vector(15 downto 0);
            SPI_EN          : in std_logic;
            FS              : out std_logic;
            SCLK            : out std_logic;
            SDO             : out std_logic;
            LDAC            : out std_logic);
   end component;

   signal SINC3_OUT, SPI_DATA_OUT, SINC3_DATA_SYNC :std_logic_vector(15 downto 0);
   signal SINC3_DONE, SPI_EN, SPI_EN_DELAY, SPI_BUSY, DONE_SYNC, CNR, NOT_SPI_BUSY : std_logic;
   signal SCK_INT, SSn_INT, SCLK_INT, FS_INT, SDO_INT, MOSI, FIFO_RDY : std_logic;
   signal LDAC_SPI, LDAC_BSP : std_logic;
begin

   T0 <= SINC3_DONE;
   T1 <= DONE_SYNC;
   T2 <= CNR;
   T3 <= SPI_EN;
   T4 <= SPI_BUSY;
   T5 <= RESN;
   T6 <= SPI_BUSY;
   T7 <= FS_INT;

   T1_2 <= '0';
   T1_3 <= '0';


   FILTER : TOP_FILTER port map(RESN       => RESN,
                                MCLK       => MCLK,
                                MDAT       => MDAT,
                                M0         => M0,
                                M1         => M1,
                                M2         => M2,
                                SINC3_OUT  => SINC3_OUT,
                                SINC3_DONE => SINC3_DONE,
                                CNR_TEST   => CNR);

   SYNC_FILTER : SINC3_SYNC port map(CLK              => CLK,
                                     RESN             => RESN, 
                                     SINC3_DATA_OUT   => SINC3_OUT,
                                     SINC3_DONE       => SINC3_DONE,
                                     SINC3_DATA_SYNC  => SINC3_DATA_SYNC,
                                     DONE_SYNC        => DONE_SYNC);



   FIFO : TOP_FIFO port map(RESN         => RESN,
                            CLK          => CLK,
                            SINC3_OUT    => SINC3_DATA_SYNC,
                            SINC3_DONE   => DONE_SYNC,
                            SPI_DATA_OUT => SPI_DATA_OUT,
                            SPI_EN       => SPI_EN,
                            SPI_EN_DELAY => SPI_EN_DELAY,
                            SPI_BUSY     => SPI_BUSY,
                            DRDY         => DRDY,
                            FS           => FIFO_RDY,
                            READ_EN      => T1_0,
                            WRITE_EN     => T1_1);
   
   SPI : SPI_MASTER port map(CLK          => CLK,
                             RESN         => RESN,
                             SPI_DATA_OUT => SPI_DATA_OUT,
                             SPI_EN       => SPI_EN,
                             SPI_EN_DELAY => SPI_EN_DELAY,
                             SPI_BUSY     => SPI_BUSY,
                             NOT_SPI_BUSY => SSn_INT,
                             SCK          => SCK_INT,
                             MOSI         => MOSI,
                             LDAC         => LDAC_SPI);

   DSP : McBSP      port map(CLK          => CLK,
                             RESN         => RESN,
                             SPI_DATA_OUT => SPI_DATA_OUT,
                             SPI_EN       => FIFO_RDY,     -- Changed (Badde)
                             FS           => FS_INT,
                             SCLK         => SCLK_INT,
                             SDO          => SDO_INT,
                             LDAC         => LDAC_BSP);
                             
                             
   process(MODE_SEL, SCK_INT, SSn_INT, SCLK_INT, SDO_INT, MOSI, FS_INT, LDAC_SPI, LDAC_BSP)
   begin
      SCK  <= SCK_INT;
      SSn  <= SSn_INT;
      SCLK <= SCLK_INT;
      if MODE_SEL = '1' then -- spi mode  -- Changed due to failure on my EVM (Badde)
         SCLK <= '0';
         SDO  <= MOSI;
         FS   <= '1';
         LDAC <= LDAC_SPI;
      elsif MODE_SEL = '0' then -- dsp mode
         SCK  <= '0';
         SSn  <= '1';
         SDO  <= SDO_INT;
         FS   <= FS_INT;
         LDAC <= LDAC_BSP;
      end if;
   end process;                             
                             
                             
end RTL;                     




library IEEE;
use IEEE.std_logic_1164.all;
library UNISIM;
use UNISIM.VComponents.ibuf;
use UNISIM.VComponents.bufg;

entity TOP2 is
   port(GCLK          : in std_logic;
        M0, M1, M2    : in std_logic;
        MODE_SEL      : in std_logic;
        MDAT1         : in std_logic;
        FS1, SCLK1    : out std_logic; 
        SCK1, SDO1    : out std_logic;
        MDAT2         : in std_logic;
        FS2, SCLK2    : out std_logic;
        SCK2, SDO2    : out std_logic;
        MCLK1         : in std_logic;
        MCLK2         : in std_logic;
        SSn1, SSn2    : out std_logic;
        LDAC1         : out std_logic;
        LDAC2         : out std_logic;
        T4, T5, T8, T9, T10,        
	   T11, T12, T13: out std_logic);
end TOP2;

architecture RTL of TOP2 is 

   component RESET_GEN
      port(CLK            : in std_logic;
           RESN_GEN       : out std_logic);
   end component;

   component TOP
      port(RESN, CLK  : in std_logic;
           M0, M1, M2 : in std_logic;
           MODE_SEL   : in std_logic;
           MDAT       : in std_logic;
           MCLK       : in std_logic;
           SCK, SSn   : out std_logic;      -- spi mode signals
           SCLK, FS   : out std_logic;  -- dsp mode signals 
           DRDY, SDO  : out std_logic;
           LDAC       : out std_logic;
           T0, T1, T2, T3, T4, T5, T6, T7 : out std_logic;
           T1_0, T1_1, T1_2, T1_3 : out std_logic);
           
   end component;

   component IBUF 
      port (O : out STD_ULOGIC; 
            I : in STD_ULOGIC); 
   end component; 

   component BUFG 
      port (O : out STD_ULOGIC; 
            I : in STD_ULOGIC); 
   end component; 

   signal T0_int1,T1_int1,T2_int1,T3_int1,T4_int1,T5_int1,T6_int1,T7_int1 : std_logic;
   signal T1_0_int1,T1_1_int1,T1_2_int1,T1_3_int1 : std_logic;

   signal T0_int2,T1_int2,T2_int2,T3_int2,T4_int2,T5_int2,T6_int2,T7_int2 : std_logic;
   signal T1_0_int2,T1_1_int2,T1_2_int2,T1_3_int2 : std_logic;

   signal DRDY1, DRDY2 : std_logic;
   signal RESN_GEN : std_logic;
   signal MCLK_INT : std_logic;
   signal MCLK_int11, MCLK_int12, MCLK_int21, MCLK_int22 : std_logic;
   signal MDATm_1, MDATm_2 : std_logic;
begin

  RESET_GEN_UNIT : RESET_GEN port map (CLK => GCLK, RESN_GEN => RESN_GEN);
  
  TOP_1_UNIT : TOP port map(RESN   => RESN_GEN,
                            CLK    => GCLK,
                            MODE_SEL => MODE_SEL,
                            M0     => M0,
                            M1     => M1,
                            M2     => M2,                           
                            MDAT   => MDAT1,
                            MCLK   => MCLK_int12,
                            SCK    => SCK1,
                            SDO    => SDO1,
                            DRDY   => DRDY1,
                            FS     => FS1,
                            SSn    => SSn1,
                            SCLK   => SCLK1,
                            LDAC   => LDAC1,
                            T0     => T4,
                            T1     => T5,
                            T2     => T8,
                            T3     => T9,
                            T4     => T12,
                            T5     => T13,
                            T6     => T6_int1,
                            T7     => T7_int1,
                            T1_0   => T4_int1,
                            T1_1   => T5_int1,
                            T1_2   => T1_2_int1,
                            T1_3   => T1_3_int1);
                            
  TOP_2_UNIT : TOP port map(RESN   => RESN_GEN,
                            CLK    => GCLK,
                            MODE_SEL => MODE_SEL,
                            M0     => M0,
                            M1     => M1,
                            M2     => M2,
                            MDAT   => MDAT2,
                            MCLK   => MCLK_int22,
                            SCK    => SCK2,
                            SDO    => SDO2,
                            DRDY   => DRDY2,
                            FS     => FS2,
                            SSn    => SSn2,
                            SCLK   => SCLK2,
                            LDAC   => LDAC2,
                            T0     => T0_int2,
                            T1     => T1_int2,
                            T2     => T2_int2,
                            T3     => T3_int2,
                            T4     => T4_int2,
                            T5     => T5_int2,
                            T6     => T6_int2,
                            T7     => T7_int2,
                            T1_0   => T1_0_int2,
                            T1_1   => T1_1_int2,
                            T1_2   => T1_2_int2,
                            T1_3   => T1_3_int2);

  IBUF_MCLK1 : IBUF port map (O => MCLK_int11, 
                              I => MCLK1);
                            
  BUFG_MCLK1 : BUFG port map (O => MCLK_int12, 
                              I => MCLK_int11); 
                              
  IBUF_MCLK2 : IBUF port map (O => MCLK_int21, 
                              I => MCLK2);
                            
  BUFG_MCLK2 : BUFG port map (O => MCLK_int22, 
                              I => MCLK_int21); 
                              
--    DRDY   <= DRDY1;
   T10 <= MCLK_int11;
   T11 <= MDAT1;
end RTL;
